@@ -2,6 +2,8 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use crate::trait_hack::YokeTraitHack;
+use crate::IsCovariant;
 use crate::Yokeable;
 use core::marker::PhantomData;
 use core::ops::Deref;
@@ -27,6 +29,9 @@ use alloc::sync::Arc;
 /// The primary constructor for [`Yoke`] is [`Yoke::attach_to_cart()`]. Several variants of that
 /// constructor are provided to serve numerous types of call sites and `Yoke` signatures.
 ///
+/// In general, `C` is a concrete type, but it is also possible for it to be a trait object;
+/// for more information, see [`IsCovariant`].
+///
 /// # Example
 ///
 /// For example, we can use this to store zero-copy deserialized data in a cache:
@@ -436,7 +441,7 @@ impl<Y: for<'a> Yokeable<'a>, C: StableDeref> Yoke<Y, Option<C>> {
         }
     }
     /// Temporary version of [`Yoke::attach_to_option_cart()`]
-    /// that doesn't hit https://github.com/rust-lang/rust/issues/84937
+    /// that doesn't hit <https://github.com/rust-lang/rust/issues/84937>
     ///
     /// See its docs for more details
     pub fn attach_to_option_cart_badly(
@@ -458,6 +463,12 @@ impl<Y: for<'a> Yokeable<'a>, C: StableDeref> Yoke<Y, Option<C>> {
 /// implemented on [`Option`] (since it's not [`Deref`]). [`CloneableCart`] essentially is
 /// "if there _is_ data to borrow from here, cloning the cart gives you an additional
 /// handle to the same data".
+///
+/// # Safety
+/// This trait is safe to implement `StableDeref` types which, once `Clone`d, point to the same underlying data.
+///
+/// (This trait is also implemented on `Option<T>` and `()`, which are the two non-`StableDeref` cart types that
+/// Yokes can be constructed for)
 pub unsafe trait CloneableCart: Clone {}
 
 #[cfg(feature = "alloc")]
@@ -468,20 +479,65 @@ unsafe impl<T: CloneableCart> CloneableCart for Option<T> {}
 unsafe impl<'a, T: ?Sized> CloneableCart for &'a T {}
 unsafe impl CloneableCart for () {}
 
-/// Clone requires that the cart derefs to the same address after it is cloned. This works for Rc, Arc, and &'a T.
-/// For all other cart types, clone `.backing_cart()` and re-use `attach_to_cart()`.
+/// Clone requires that the cart derefs to the same address after it is cloned. This works for
+/// Rc, Arc, and &'a T.
+///
+/// For other cart types, clone `.backing_cart()` and re-use `.attach_to_cart()`; however, doing
+/// so may lose mutations performed via `.with_mut()`.
+///
+/// Cloning a `Yoke` is often a cheap operation requiring no heap allocations, in much the same
+/// way that cloning an `Rc` is a cheap operation. However, if the `yokeable` contains owned data
+/// (e.g., from `.with_mut()`), that data will need to be cloned.
 impl<Y: for<'a> Yokeable<'a>, C: CloneableCart> Clone for Yoke<Y, C>
 where
-    for<'a> <Y as Yokeable<'a>>::Output: Clone,
+    for<'a> YokeTraitHack<<Y as Yokeable<'a>>::Output>: Clone,
 {
     fn clone(&self) -> Self {
+        let this: &Y::Output = self.get();
+        // We have an &T not a T, and we can clone YokeTraitHack<T>
+        let this_hack = YokeTraitHack(this).into_ref();
         Yoke {
-            yokeable: unsafe { Y::make(self.get().clone()) },
+            yokeable: unsafe { Y::make(this_hack.clone().0) },
             cart: self.cart.clone(),
         }
     }
 }
 
+// This is safe because Y is 'static and C has a covariant lifetime
+unsafe impl<'b, Y: for<'a> Yokeable<'a>, C: IsCovariant<'b>> IsCovariant<'b> for Yoke<Y, C> {}
+
+#[test]
+fn test_clone() {
+    let local_data = "foo".to_string();
+    let y1 = Yoke::<alloc::borrow::Cow<'static, str>, Rc<String>>::attach_to_rc_cart(Rc::new(
+        local_data,
+    ));
+
+    // Test basic clone
+    let y2 = y1.clone();
+    assert_eq!(y1.get(), "foo");
+    assert_eq!(y2.get(), "foo");
+
+    // Test clone with mutation on target
+    let mut y3 = y1.clone();
+    y3.with_mut(|y| {
+        y.to_mut().push_str("bar");
+    });
+    assert_eq!(y1.get(), "foo");
+    assert_eq!(y2.get(), "foo");
+    assert_eq!(y3.get(), "foobar");
+
+    // Test that mutations on source do not affect target
+    let y4 = y3.clone();
+    y3.with_mut(|y| {
+        y.to_mut().push_str("baz");
+    });
+    assert_eq!(y1.get(), "foo");
+    assert_eq!(y2.get(), "foo");
+    assert_eq!(y3.get(), "foobarbaz");
+    assert_eq!(y4.get(), "foobar");
+}
+
 impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     /// Allows one to "project" a yoke to perform a transformation on the data, potentially
     /// looking at a subfield, and producing a new yoke. This will move cart, and the provided
@@ -499,8 +555,10 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     ///
     /// This can be used, for example, to transform data from one format to another:
     ///
+    /// ***[#1061](https://github.com/unicode-org/icu4x/issues/1061): The following example
+    /// requires Rust 1.56.***
+    ///
     /// ```rust,ignore
-    /// # // This doctest is temporarily ignored because of https://github.com/rust-lang/rust/issues/86703
     /// # use std::rc::Rc;
     /// # use yoke::Yoke;
     /// #
@@ -512,8 +570,10 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
     ///
     /// This can also be used to create a yoke for a subfield
     ///
+    /// ***[#1061](https://github.com/unicode-org/icu4x/issues/1061): The following example
+    /// requires Rust 1.56.***
+    ///
     /// ```rust,ignore
-    /// # // This doctest is temporarily ignored because of https://github.com/rust-lang/rust/issues/86703
     /// # use std::borrow::Cow;
     /// # use yoke::{Yoke, Yokeable};
     /// # use std::mem;
@@ -645,6 +705,51 @@ impl<Y: for<'a> Yokeable<'a>, C> Yoke<Y, C> {
             cart: self.cart.clone(),
         }
     }
+
+    /// A version of [`Yoke::project`] that takes a capture and bubbles up an error
+    /// from the callback function.
+    #[allow(clippy::type_complexity)]
+    pub fn try_project_with_capture<P, T, E>(
+        self,
+        capture: T,
+        f: for<'a> fn(
+            <Y as Yokeable<'a>>::Output,
+            capture: T,
+            PhantomData<&'a ()>,
+        ) -> Result<<P as Yokeable<'a>>::Output, E>,
+    ) -> Result<Yoke<P, C>, E>
+    where
+        P: for<'a> Yokeable<'a>,
+    {
+        let p = f(self.yokeable.transform_owned(), capture, PhantomData)?;
+        Ok(Yoke {
+            yokeable: unsafe { P::make(p) },
+            cart: self.cart,
+        })
+    }
+
+    /// A version of [`Yoke::project_cloned`] that takes a capture and bubbles up an error
+    /// from the callback function.
+    #[allow(clippy::type_complexity)]
+    pub fn try_project_cloned_with_capture<'this, P, T, E>(
+        &'this self,
+        capture: T,
+        f: for<'a> fn(
+            &'this <Y as Yokeable<'a>>::Output,
+            capture: T,
+            PhantomData<&'a ()>,
+        ) -> Result<<P as Yokeable<'a>>::Output, E>,
+    ) -> Result<Yoke<P, C>, E>
+    where
+        P: for<'a> Yokeable<'a>,
+        C: CloneableCart,
+    {
+        let p = f(self.get(), capture, PhantomData)?;
+        Ok(Yoke {
+            yokeable: unsafe { P::make(p) },
+            cart: self.cart.clone(),
+        })
+    }
 }
 
 /// Safety docs for project()
